@@ -31,7 +31,7 @@
 #' \item{WT:}  {Amino acid found at that position in the wild-type protein.}
 #' \item{NW:}  {New amino acid found in the mutated protein.}
 #' \item{DDG:} {Change in Gibbs free energy (kcal/mol), defined as DDG = DGmt - DGwt, where DG is the change in Gibbs free energy for the folding of the protein from its unfolded state. Thus, a positive value means a stabilizing effect, and vice versa.}
-#' \item{pH:}  {-log[H+]}
+#' \item{pH:}  {-log H+]}
 #' \item{T:}   {Temperature in Celsius degrees.}
 #' \item{RSA:}  {Relative Solvent Accessible Area (Only if a PDB file has been provided).}
 #' }
@@ -43,6 +43,7 @@
 #' @importFrom httr http_error
 #' @importFrom httr content
 #' @importFrom xml2 read_html
+#' @importFrom xml2 xml_text
 #' @importFrom bio3d get.pdb
 #' @export
 
@@ -55,7 +56,7 @@ imutant <- function(protein, ch = "_", pos, newres = "",
 
 
   ## ----------------- Checking paramaters ---------------------- ##
-  if (nchar(protein) ==4){
+  if (nchar(protein) == 4){
     is.pdb <- TRUE
     pdb <- paste(protein, ch, sep = ":")
     nc <- 7 # number variable of the output dataframe
@@ -90,10 +91,18 @@ imutant <- function(protein, ch = "_", pos, newres = "",
                            "ph" = as.character(pH),
                            "pred" = "ddg")
 
-  resp <- httr::POST(cgi_url, body = query_parameters)
-
-  if (httr::http_error(resp)){
-    stop(paste("HTTP communication failure: ", resp$status_code, sep = ""))
+  resp <- tryCatch(
+    {
+      httr::POST(cgi_url, body = query_parameters)
+    },
+    error = function(cond){
+      return(NULL)
+    },
+    warning = function(w) conditionMessage(w)
+  )
+  if (is.null(resp)){
+    message("Sorry, the server did not respond")
+    return(NULL)
   } else {
     resp_xml <- xml2::read_html(resp)
   }
@@ -102,6 +111,10 @@ imutant <- function(protein, ch = "_", pos, newres = "",
   a <- xml2::xml_find_all(resp_xml, xpath = "//a")
   b <- xml2::xml_text(a)
   url <- b[sapply(b, function(x) grepl("https://folding", x))]
+  if (length(url) == 0){
+    message("Sorry, no url for the response could be retrieved")
+    return(NULL)
+  }
 
   ## ------------------ Parsing the result page ----------------- ##
   waiting <- TRUE
@@ -118,6 +131,11 @@ imutant <- function(protein, ch = "_", pos, newres = "",
     }
   }
 
+  if (counter >= times){
+    message("Sorry timeout waiting the server's response")
+    return(NULL)
+  }
+
   resource_ <- httr::content(resource)
   text <- xml2::xml_text(resource_)
   t <- strsplit(text, "\n")[[1]]
@@ -162,7 +180,7 @@ imutant <- function(protein, ch = "_", pos, newres = "",
 #' @details Two computational approaches for prediction of the effect of amino acid changes on protein stability are implemented. FoldX (buildmodel and positionscan methods) uses a force field approach and although it has been proved to be satisfactorily accurate, it is also a time-consuming method. An alternative much faster is I-Mutant, a method base on machine-learning
 #' @return The function computes and returns the DDG (kcal/mol) for the requested residue change, defined as DDG = DGmt - DGwt, where DG is the Gibbs free energy for the folding of the protein from its unfolded state. Thus, a positive value means a destabilizing effect, and vice versa.
 #' @author Juan Carlos Aledo
-#' @examples \dontrun{foldx('1aaq', 'A', 45, 'R')}
+#' @examples \dontrun{foldx.mut('1aaq', 'A', 45, 'R')}
 #' @seealso imutant(), ddG.profile()
 #' @references Schymkowitz et al (2005) Nucl. Ac. Res. 33:W382-W388.
 #' @importFrom bio3d get.pdb
@@ -191,7 +209,20 @@ foldx.mut <- function(pdb, ch, pos, newres = "", pH = 7,
   # start repairing the input PDB (see http://foldxsuite.crg.eu/command/RepairPDB).
 
   if (nchar(pdb) == 4){
-    bio3d::get.pdb(pdb)
+    x <- tryCatch(
+      {
+        bio3d::get.pdb(pdb)
+      },
+      error = function(cond){
+        return(NULL)
+      },
+      warning = function(w) conditionMessage(w)
+    )
+    if (is.null(x) | grepl("cannot", x)){
+      message("Sorry, get.pdb failed")
+      file.remove(paste(pdb, ".pdb", sep = ""))
+      return(NULL)
+    }
     path <- "./"
     id <- pdb
   } else {
@@ -201,19 +232,31 @@ foldx.mut <- function(pdb, ch, pos, newres = "", pH = 7,
     if (grepl("_Repair", id)){
       id <- strsplit(id, "_")[[1]][1]
     }
+    oldwd <- getwd()
+    on.exit(setwd(oldwd))
+    setwd(path)
   }
 
-  oldwd <- getwd()
-  on.exit(setwd(oldwd))
-  setwd(path)
-
   repaired = FALSE
-  if (!grepl("_Repair.pdb", path)){
+  if (!grepl("_Repair.pdb", pdb)){
     repaired = TRUE
     A <- paste("foldx --command=RepairPDB --pdb=", pdb, ".pdb --ionStrength=0.05 ",  sep = "")
     B <- paste("--pH=", pH, " --water=CRYSTAL --vdwDesign=2 --pdbHydrogen=false", sep = "")
     AB <- paste(A, B)
-    system( AB)
+
+    x <- tryCatch(
+      {
+        system(AB)
+      },
+      error = function(cond){
+        return(NULL)
+      },
+      warning = function(w) conditionMessage(w)
+    )
+    if (is.null(x) | grepl("error", x) | x >0){
+      message("Sorry, foldx failed")
+      return(NULL)
+    }
   }
   ## --- Cleaning unnecessary files generated by FoldX --- ##
   if (repaired){
@@ -225,14 +268,32 @@ foldx.mut <- function(pdb, ch, pos, newres = "", pH = 7,
   if (method == 'buildmodel'){ ## ------------------------------ Method FoldX-BuildModel
 
     ## -------------- Formatting the mutations -------------- ##
+    mypdb <- NULL
     if (file.exists(pdb)){
       mypdb <- bio3d::read.pdb(pdb)$atom
     } else {
       mypdb <- bio3d::read.pdb(paste(id, "_Repair.pdb", sep = ""))$atom
     }
+    if (is.null(mypdb)){
+      message("Sorry, ddG.mut failed")
+      return(NULL)
+    }
 
     wtres <- bio3d::aa321(mypdb$resid[which(mypdb$resno == pos & mypdb$chain == ch)[1]])
-    con <- file('individual_list.txt', 'w')
+
+    con <- tryCatch(
+      {
+        file('individual_list.txt', 'w')
+      },
+      error = function(cond){
+        return(NULL)
+      }
+    )
+    if (is.null(con) | grepl("No such file", con)){
+      message("Sorry, ddG.mut failed")
+      return(NULL)
+    }
+
     if (newres %in% aa){
       writeLines(paste(wtres, ch, pos, newres, ';', sep = ""), con)
     } else {
@@ -253,10 +314,35 @@ foldx.mut <- function(pdb, ch, pos, newres = "", pH = 7,
     B <- paste(" --mutant-file=individual_list.txt --ionStrength=0.05 --pH=", pH,sep = "")
     C <- " --water=CRYSTAL --vdwDesign=2 --pdbHydrogens=false --numberOfRuns=3"
     ABC <- paste(A, B, C)
-    system(ABC)
+
+    x <- tryCatch(
+      {
+        system(ABC)
+      },
+      error = function(cond){
+        return(NULL)
+      },
+      warning = function(w) conditionMessage(w)
+    )
+    if (is.null(x) | grepl("error", x) | x >0){
+      message("Sorry, foldx failed")
+      return(NULL)
+    }
+
 
     ## -------------- Parsing the result page -------------- ##
-    con <- file(paste('Average_', id, "_Repair.fxout", sep = ""), 'r')
+    con <- tryCatch(
+      {
+        file(paste('Average_', id, "_Repair.fxout", sep = ""), 'r')
+      },
+      error = function(cond){
+        return(NULL)
+      }
+    )
+    if (is.null(con) | grepl("No such file", con)){
+      message("Sorry, ddG.mut failed")
+      return(NULL)
+    }
     lines <- readLines(con)
     close(con)
 
@@ -289,7 +375,8 @@ foldx.mut <- function(pdb, ch, pos, newres = "", pH = 7,
         df$SD[i] <- as.numeric(l[2])
       }
     } else {
-      stop("Something went wrong when parsing the result file")
+      message("Something went wrong when parsing the result file")
+      return(NULL)
     }
 
     ## --- Cleaning unnecessary files generated by FoldX --- ##
@@ -319,6 +406,7 @@ foldx.mut <- function(pdb, ch, pos, newres = "", pH = 7,
     if (!is.null(war)){
       warning(war)
     }
+    closeAllConnections()
     return(df)
 
   } else if (method == 'positionscan'){ ## -------------------- Method FoldX-PositionScan
@@ -338,11 +426,16 @@ foldx.mut <- function(pdb, ch, pos, newres = "", pH = 7,
     # Charged NE2 His .... H2S ... e
     # Neutral His ........ H3S ... f
     ## -------------- Formating the mutations -------------- ##
+    mypdb <- NULL
     if (file.exists(pdb)){
       mypdb <- bio3d::read.pdb(pdb)$atom
     } else {
       mypdb <- bio3d::read.pdb(paste(id, "_Repair.pdb", sep = ""))$atom
     }
+    if (is.null(mypdb)){
+      message("Sorry, ddG.mut failed")
+      return(NULL)
+    }
 
     wtres <- bio3d::aa321(mypdb$resid[which(mypdb$resno == pos & mypdb$chain == ch)[1]])
     if (newres %in% c(aa, nst_aa)){
@@ -354,9 +447,35 @@ foldx.mut <- function(pdb, ch, pos, newres = "", pH = 7,
     A <- paste("foldx --command=PositionScan --pdb=", id, "_Repair.pdb", sep = "")
     B <- paste(" --positions=", positions, sep = "")
     AB <- paste(A, B)
-    system(AB)
+
+    x <- tryCatch(
+      {
+        system(AB)
+      },
+      error = function(cond){
+        return(NULL)
+      },
+      warning = function(w) conditionMessage(w)
+    )
+    if (is.null(x) | grepl("error", x) | x >0){
+      message("Sorry, foldx failed")
+      return(NULL)
+    }
+
     ## -------------- Parsing the result page -------------- ##
-    con <- file(paste('PS_', id, "_Repair_scanning_output.txt", sep = ""), 'r')
+    con <- tryCatch(
+      {
+        file(paste('PS_', id, "_Repair_scanning_output.txt", sep = ""), 'r')
+      },
+      error = function(cond){
+        return(NULL)
+      }
+    )
+    if (is.null(con) | grepl("No such file", con)){
+      message("Sorry, ddG.mut failed")
+      return(NULL)
+    }
+
     lines <- readLines(con)
     close(con)
     ## --------------- Output dataframe -------------------- ##
@@ -389,7 +508,8 @@ foldx.mut <- function(pdb, ch, pos, newres = "", pH = 7,
         df$SD[i] <- NA
       }
     } else {
-      stop("Something went wrong when parsing the result file")
+      message("Something went wrong when parsing the result file")
+      return(NULL)
     }
 
     if (!keepfiles){
@@ -414,9 +534,9 @@ foldx.mut <- function(pdb, ch, pos, newres = "", pH = 7,
     if (!is.null(war)){
       warning(war)
     }
+    closeAllConnections()
     return(df)
   }
-
 }
 
 
@@ -443,7 +563,20 @@ foldx.stab <- function(pdb, pH = 7, I = 0.05) {
 
   ## --------- Getting the PDB ----------- ##
   if (nchar(pdb) == 4){
-    bio3d::get.pdb(pdb)
+    x <- tryCatch(
+      {
+        bio3d::get.pdb(pdb)
+      },
+      error = function(cond){
+        return(NULL)
+      },
+      warning = function(w) conditionMessage(w)
+    )
+    if (is.null(x) | grepl("cannot", x)){
+      message("Sorry, get.pdb failed")
+      file.remove(paste(pdb, ".pdb", sep = ""))
+      return(NULL)
+    }
     path <- "./"
     id <- pdb
   } else {
@@ -463,14 +596,41 @@ foldx.stab <- function(pdb, pH = 7, I = 0.05) {
     A <- paste("foldx --command=RepairPDB --pdb=", id, ".pdb --ionStrength=", I,  sep = "")
     B <- paste(" --pH=", pH, " --water=CRYSTAL --vdwDesign=2 --pdbHydrogen=false", sep = "")
     AB <- paste(A, B)
-    system(AB)
+
+    x <- tryCatch(
+      {
+        system(AB)
+      },
+      error = function(cond){
+        return(NULL)
+      },
+      warning = function(w) conditionMessage(w)
+    )
+    if (is.null(x) | grepl("error", x) | x >0){
+      message("Sorry, foldx failed")
+      return(NULL)
+    }
+
   }
 
   ## ------- Folding energy computation ------- ##
   A <- paste("foldx --command=Stability --pdb=", id, "_Repair.pdb", sep = "")
   B <- paste(" --pdb-dir=", path, " --output-dir=", path, sep = "")
   AB <- paste(A, B)
-  system(AB)
+
+  x <- tryCatch(
+    {
+      system(AB)
+    },
+    error = function(cond){
+      return(NULL)
+    },
+    warning = function(w) conditionMessage(w)
+  )
+  if (is.null(x) | grepl("error", x) | x >0){
+    message("Sorry, foldx failed")
+    return(NULL)
+  }
 
   ## ---------- Parsing the results ---------- ##
   DG <- read.csv2(file = paste(path, "/", id, "_Repair_0_ST.fxout", sep = ""), header = FALSE, sep = "\t")
@@ -496,6 +656,7 @@ foldx.stab <- function(pdb, pH = 7, I = 0.05) {
   ## --------- Output ----------- ##
   attr(DG, "pdb") <- pdb
   attr(DG, "units") <- "kcal/mol"
+  closeAllConnections()
   return(DG)
 }
 
@@ -518,13 +679,28 @@ foldx.stab <- function(pdb, pH = 7, I = 0.05) {
 #' @seealso foldx.stab()
 #' @references Schymkowitz et al (2005) Nucl. Ac. Res. 33:W382-W388.
 #' @importFrom bio3d get.pdb
+#' @importFrom stats complete.cases
 #' @export
 
 foldx.assembly <- function(pdb, mol1, mol2, pH = 7, I = 0.05) {
 
   ## --------- Getting the PDB ----------- ##
   if (nchar(pdb) == 4){
-    bio3d::get.pdb(pdb)
+
+    x <- tryCatch(
+      {
+        bio3d::get.pdb(pdb)
+      },
+      error = function(cond){
+        return(NULL)
+      },
+      warning = function(w) conditionMessage(w)
+    )
+    if (is.null(x) | grepl("cannot", x)){
+      message("Sorry, get.pdb failed")
+      file.remove(paste(pdb, ".pdb", sep = ""))
+      return(NULL)
+    }
     path <- "./"
     id <- pdb
   } else {
@@ -544,14 +720,38 @@ foldx.assembly <- function(pdb, mol1, mol2, pH = 7, I = 0.05) {
     A <- paste("foldx --command=RepairPDB --pdb=", id, ".pdb --ionStrength=", I,  sep = "")
     B <- paste(" --pH=", pH, " --water=CRYSTAL --vdwDesign=2 --pdbHydrogen=false", sep = "")
     AB <- paste(A, B)
-    system(AB)
+    x <- tryCatch(
+      {
+        system(AB)
+      },
+      error = function(cond){
+        return(NULL)
+      },
+      warning = function(w) conditionMessage(w)
+    )
+    if (is.null(x) | grepl("error", x) | x >0){
+      message("Sorry, foldx failed")
+      return(NULL)
+    }
   }
 
   ## ------ Complex Analysis --------------- ##
   A <- paste("foldx --command=AnalyseComplex --pdb=", id, "_Repair.pdb --analyseComplexChains=", mol1, ",", mol2, sep = "")
   B <- paste(" --pdb-dir=", path, " --output-dir=", path, sep = "")
   AB <- paste(A, B, sep = "")
-  system(AB)
+  x <- tryCatch(
+    {
+      system(AB)
+    },
+    error = function(cond){
+      return(NULL)
+    },
+    warning = function(w) conditionMessage(w)
+  )
+  if (is.null(x) | grepl("error", x) | x >0){
+    message("Sorry, foldx failed")
+    return(NULL)
+  }
 
   ## ---------- Parsing the results ---------- ##
   # a <- read.csv2(file = "./Indiv_energies_ABC_Repair_AC.fxout", skip = 8, sep = "\t", fill=TRUE)
@@ -614,6 +814,7 @@ foldx.assembly <- function(pdb, mol1, mol2, pH = 7, I = 0.05) {
   attr(IR, "Gibbs") <- DG
   attr(IR, "units") <- "kcal/mol"
   attr(IR, "pdb") <- pdb
+  closeAllConnections()
   return(IR)
 }
 
@@ -646,7 +847,7 @@ ddG.profile <- function(prot, ch, pos, pH = 7, Te = 25){
 
   ## ----------------- Checking paramaters ---------------------- ##
   war <- NULL
-  if (nchar(prot) ==4){
+  if (nchar(prot) == 4){
     is.pdb <- TRUE
     pdb <- paste(prot, ch, sep = ":")
     nc <- 7 # number variable of the output dataframe
@@ -670,7 +871,20 @@ ddG.profile <- function(prot, ch, pos, pH = 7, Te = 25){
     stop("The argument 'pos' must be a valid numeric position within the primary structure of the protein")
   }
 
-  t <- imutant(protein = prot, ch = ch, pos = pos, pH = pH, Te = Te)
+  t <- tryCatch(
+    {
+      imutant(protein = prot, ch = ch, pos = pos, pH = pH, Te = Te)
+    },
+    error = function(cond){
+      return(NULL)
+    }
+  )
+  if (is.null(t)){
+    message("Sorry, imutant failed")
+    return(NULL)
+  }
+
+
   ddG <- t$DDG
   names(ddG) <- t$NEW
 
@@ -744,7 +958,20 @@ ddG.ptm <- function(pdb, ch, pos, ptm, dir = 'f', pH = 7){
 
   ## --------- Getting the PDB ----------- ##
   if (nchar(pdb) == 4){
-    bio3d::get.pdb(pdb)
+    x <- tryCatch(
+      {
+        bio3d::get.pdb(pdb)
+      },
+      error = function(cond){
+        return(NULL)
+      },
+      warning = function(w) conditionMessage(w)
+    )
+    if (is.null(x) | grepl("cannot", x)){
+      message("Sorry, get.pdb failed")
+      file.remove(paste(pdb, ".pdb", sep = ""))
+      return(NULL)
+    }
     path <- "./"
     id <- pdb
   } else {
@@ -773,7 +1000,20 @@ ddG.ptm <- function(pdb, ch, pos, ptm, dir = 'f', pH = 7){
     A <- paste("foldx --command=RepairPDB --pdb=", id, ".pdb --ionStrength=0.05 ",  sep = "")
     B <- paste("--pH=", pH, " --water=CRYSTAL --vdwDesign=2 --pdbHydrogen=false", sep = "")
     AB <- paste(A, B)
-    system(AB)
+    x <- tryCatch(
+      {
+        system(AB)
+      },
+      error = function(cond){
+        return(NULL)
+      },
+      warning = function(w) conditionMessage(w)
+    )
+    if (is.null(x) | grepl("error", x) | x >0){
+      message("Sorry, foldx failed")
+      return(NULL)
+    }
+
   }
 
 
@@ -846,7 +1086,20 @@ ddG.ptm <- function(pdb, ch, pos, ptm, dir = 'f', pH = 7){
   B <- " --mutant-file=individual_list.txt --ionStrength=0.05 --pH=7 --water=CRYSTAL"
   C <- " --vdwDesign=2 --pdbHydrogens=false --numberOfRuns=1"
   ABC <- paste(A,B,C, sep = "")
-  system(ABC)
+  x <- tryCatch(
+    {
+      system(ABC)
+    },
+    error = function(cond){
+      return(NULL)
+    },
+    warning = function(w) conditionMessage(w)
+  )
+  if (is.null(x) | grepl("error", x) | x >0){
+    message("Sorry, foldx failed")
+    return(NULL)
+  }
+
 
   ## -------------- Parsing the result page -------------- ##
   result_file <- paste('Average_', id, "_Repair.fxout", sep = "")
@@ -902,6 +1155,7 @@ ddG.ptm <- function(pdb, ch, pos, ptm, dir = 'f', pH = 7){
   attr(output, "PTM") <- ptm
   attr(output, "units") <- "kcal/mol"
 
+  closeAllConnections()
   return(output)
 
 }
